/*
 * fontx2import.c
 * original code:SASANO Takayoshi
 * --- public domain.
 */

#include <basic.h>
#include <errcode.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "jis2sjis.h"
#include "fontx2defs.h"


/* FONTX2$B7A<0$+$I$&$+$N%A%'%C%/(B */
EXPORT Bool	fontx2_isfontx2(VP fontdata)
{
    FONTX2_HEADER	*h = fontdata;

    return !memcmp("FONTX2", h->identifier, sizeof(h->identifier));
}

/* $B4A;z%U%)%s%H%G!<%?$+$I$&$+$N%A%'%C%/(B */
EXPORT Bool	fontx2_iskanjifont(VP fontdata)
{
    FONTX2_HEADER	*h = fontdata;

    return h->codetype == CODETYPE_KANJI;
}

/* $B1Q;z%U%)%s%H%G!<%?$+$I$&$+$N%A%'%C%/(B */
EXPORT Bool	fontx2_isasciifont(VP fontdata)
{
    FONTX2_HEADER	*h = fontdata;

    return h->codetype == CODETYPE_ASCII;
}

/* JIS$B%3!<%I"*(Bfont index$B$NJQ49!J(BASCII$B%3!<%I$O(BASCII$B%3!<%I$,(Bindex$B$H$J$k!K(B */
EXPORT WERR	fontx2_getkanjifontindex(VP fontdata, W jis)
{
    FONTX2_KANJI	*f = fontdata;
    W	i, sjis, start, end, index;

    sjis = _mbcjistojms(jis);			/* SJIS$B$G=hM}$9$kI,MW$,$"$k(B */

    index = 0;
    for(i = 0; i < f->tnum; i++) {
	start = f->block[i].start;
	end   = f->block[i].end;

	if(sjis >= start && sjis <= end) {	/* $B$"$C$?(B */
	    index += sjis - start;
	    break;
	}

	index += end - start + 1;		/* $B$J$+$C$?"*<!$N%V%m%C%/$X(B */
    }

    return (i == f->tnum) ? ER_PAR : index;
}

/* 1$BJ8;z$N%U%)%s%H$r9=@.$9$k%P%$%H?t$r<hF@$9$k(B */
EXPORT W	fontx2_getcharsize(VP fontdata, W *xsize, W *ysize)
{
    FONTX2_HEADER	*h = fontdata;

    if(xsize != NULL) *xsize = h->xsize;
    if(ysize != NULL) *ysize = h->ysize;

    return h->xsize * h->ysize / 8;
}

/* 1$BJ8;zJ,$N%U%)%s%H%G!<%?$,B8:_$9$k%]%$%s%?$r<hF@$9$k(B */
EXPORT UB	*fontx2_getcharptr(VP fontdata, W index, W size)
{
    FONTX2_HEADER	*h = fontdata;
    FONTX2_ASCII	*a = fontdata;
    FONTX2_KANJI	*k = fontdata;

    switch(h->codetype) {
    case	CODETYPE_ASCII:
	return &a->d[size * index];
    case	CODETYPE_KANJI:
	return (UB *)&k->block[k->tnum] + size * index;
    default:
	return NULL;
    }
}
